#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include "winsrl.h"


// Forward Declarations
BOOL ApplySerialSettings(PSERIAL_CONNECTION pSerialConnection);

//---------------------------------------------------------------------------
//	Description:
//		This routines Builds a default serial connection structure
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//
//	The default serial settings are as follows
//
//	iComPortNumber			= 1;			//	Serial port number
//	dwReceiveBufferSize		= 4096;			//	Required receive buffer size of driver
//	dwTransmitBufferSize	= 4096;			//	Required transmit buffer size of driver
//	bByteSize				= 8;			//	Byte Size range(4-8)
//	bFlowCtrl				= FC_NONE;		//	Flow Control
//	bParity					= NOPARITY;		//	Parity
//	bStopBits				= ONESTOPBIT;	//	Stop Bits
//	dwBaudRate				= CBR_9600;		//	Baud rate
//	dwTimeOutMultiplier		= 1;			//	This multiplied by the number of bytes will
//											//	be the timeout in milliseconds for read or write operations
//
//---------------------------------------------------------------------------

void BuildDefaultSerialConnectionStruct(PSERIAL_CONNECTION pSerialConnection)
{
	pSerialConnection->iComPortNumber			= 1;
	pSerialConnection->dwReceiveBufferSize		= 4096;
	pSerialConnection->dwTransmitBufferSize		= 4096;
	pSerialConnection->bByteSize				= 8;
	pSerialConnection->bFlowCtrl				= FC_NONE;
	pSerialConnection->bParity					= NOPARITY;
	pSerialConnection->bStopBits				= ONESTOPBIT;
	pSerialConnection->dwBaudRate				= CBR_9600;
	pSerialConnection->dwTimeOutMultiplier		= 2;
}

//---------------------------------------------------------------------------
//	Description:
//		This routines tries to open a connection to the comm
//		port and applies the settings
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//	Outputs:
//		True if operation succsessful otherwise false
//
//---------------------------------------------------------------------------

BOOL OpenComPort(PSERIAL_CONNECTION pSerialConnection)
{
	char ComPortName[10];
	COMMTIMEOUTS CommTimeOuts;
//	COMMPROP CommProp;

	sprintf(ComPortName, "COM%d", pSerialConnection->iComPortNumber);
	// Open handle to COMM device
	pSerialConnection->idComDev = CreateFile( ComPortName, GENERIC_READ | GENERIC_WRITE, 0, NULL,
													OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL );
	// Check if COMM device handle is valid
	if( pSerialConnection->idComDev==INVALID_HANDLE_VALUE)
	{
		if (PRINT_DEBUG_INFO)
		{
			printf("\n\n\nERROR: Could not open Serial port device handle\n\n\n");
		}
		return FALSE ;
	}

	// Perform onetime initializations
	
	// Setup device buffers
	SetupComm( pSerialConnection->idComDev, pSerialConnection->dwReceiveBufferSize, 
											pSerialConnection->dwTransmitBufferSize );

/*	if(GetCommProperties(pSerialConnection->idComDev, &CommProp))
	{
		pSerialConnection->dwReceiveBufferSize = CommProp.dwCurrentRxQueue;
		pSerialConnection->dwTransmitBufferSize = CommProp.dwCurrentTxQueue;
	}
*/

	// Purge any information in the buffer
	PurgeComm( pSerialConnection->idComDev, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ) ;
	// Set up Comm port timeouts for read and write
	CommTimeOuts.ReadIntervalTimeout = 0 ;
	CommTimeOuts.ReadTotalTimeoutMultiplier = pSerialConnection->dwTimeOutMultiplier ;
	CommTimeOuts.ReadTotalTimeoutConstant = 0 ;
	// CBR_9600 is approximately 1byte/ms. For our purposes, allow
	// double the expected time per character for a fudge factor.
	CommTimeOuts.WriteTotalTimeoutMultiplier = pSerialConnection->dwTimeOutMultiplier;
	CommTimeOuts.WriteTotalTimeoutConstant = 0 ;
	SetCommTimeouts( pSerialConnection->idComDev, &CommTimeOuts ) ;

	//	Apply other settings
	ApplySerialSettings(pSerialConnection);
	return TRUE;
}


//---------------------------------------------------------------------------
//	Description:
//		This routines sets up the DCB based on settings 
//		and performs a SetCommState().
//		This function needs to be called only if settings
//		are changed after OpenCommPort function call.
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//	Outputs:
//		True if operation succsessful otherwise false
//
//---------------------------------------------------------------------------
BOOL ApplySerialSettings(PSERIAL_CONNECTION pSerialConnection)
{
	BYTE bSet ;
	DCB  dcb ;

	if( pSerialConnection->idComDev==INVALID_HANDLE_VALUE)
		return FALSE;

	dcb.DCBlength = sizeof( DCB ) ;

	GetCommState( pSerialConnection->idComDev, &dcb ) ;

	dcb.BaudRate = pSerialConnection->dwBaudRate ;
	dcb.ByteSize = pSerialConnection->bByteSize ;
	dcb.Parity = pSerialConnection->bParity ;
	dcb.StopBits = pSerialConnection->bStopBits ;

	// setup hardware flow control
	bSet = (BYTE) ((pSerialConnection->bFlowCtrl & FC_DTRDSR) != 0) ;
	dcb.fOutxDsrFlow = bSet ;
	if (bSet) 
		dcb.fDtrControl = DTR_CONTROL_HANDSHAKE ;
	else
		dcb.fDtrControl = DTR_CONTROL_ENABLE ;

	bSet = (BYTE) ((pSerialConnection->bFlowCtrl & FC_RTSCTS) != 0) ;
	dcb.fOutxCtsFlow = bSet ;
	if (bSet)
		dcb.fRtsControl = RTS_CONTROL_HANDSHAKE ;
	else
		dcb.fRtsControl = RTS_CONTROL_ENABLE ;

	// setup software flow control

	bSet = (BYTE) ((pSerialConnection->bFlowCtrl & FC_XONXOFF) != 0) ;
	dcb.fInX = dcb.fOutX = bSet ;
	dcb.XonChar = ASCII_XON ;
	dcb.XoffChar = ASCII_XOFF ;
	dcb.XonLim = 100 ;
	dcb.XoffLim = 100 ;

	// other various settings
	dcb.fBinary = TRUE ;
	dcb.fParity = TRUE ;

	if (!SetCommState( pSerialConnection->idComDev, &dcb ))
	{
		if (PRINT_DEBUG_INFO)
		{
			printf("\n\n\nERROR: Couldnot apply settings to serial port number %d\n", 
																	pSerialConnection->iComPortNumber);
			printf("Serial driver returned error code %ld\n\n\n", GetLastError());
		}
		return FALSE;
	}

	return TRUE;
} // end of SetupConnection()

//---------------------------------------------------------------------------
//	Description:
//		Reads a block from the COM port and stuffs it into
//		the provided buffer.
//
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//		OVERLAPPED  pSerialConnection->osRead
//			Structure used for overlapped read
//		LPSTR lpszBlock
//			block used for storage of read bytes
//		DWORD dwBytesToBeRead
//			Number of bytes to be read
//	Outputs:
//		Number of bytes of data received from the comm port
//
//---------------------------------------------------------------------------

DWORD ReadCommBlock( PSERIAL_CONNECTION pSerialConnection, LPSTR lpszBlock, DWORD dwBytesToBeRead )
{
	DWORD dwBytesRead, i;
   
	if (dwBytesToBeRead >= pSerialConnection->dwReceiveBufferSize)
	{
		if (PRINT_DEBUG_INFO)
		{
			printf("\n\n\nINVALID PARAMETER: Cannot receive more than %d characters in one read operation\n", 
																	pSerialConnection->dwReceiveBufferSize);
			printf("Break up reads into smaller chunks\n\n\n");
		}
		return 0;
	}
	
	if(!ReadFile( pSerialConnection->idComDev, lpszBlock, dwBytesToBeRead, &dwBytesRead, NULL))
	{
		if (PRINT_DEBUG_INFO)
		{
			printf("\n\n\nERROR: Readfile returned error code %d\n\n\n", GetLastError());
		}
		return 0;
	}
	if (dwBytesRead < dwBytesToBeRead)
	{
		if (PRINT_DEBUG_INFO)
		{
			printf("\n\n\nREAD TIMED-OUT: Could read only %d of %d bytes\n\n\n", dwBytesRead, dwBytesToBeRead);
		}
	}
	if (PRINT_DEBUG_INFO)
	{
		for(i=0; i<dwBytesRead; i++)
			printf("READ : 0X%x\n", *(lpszBlock+i));
	}
	return dwBytesRead;
} // end of ReadCommBlock()

//---------------------------------------------------------------------------
//  Writes a block of data to the COM port
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//		LPSTR lpByte
//			pointer to data to write to port
//		DWORD dwBytesToBeWritten
//			Number of bytes to be written
//	Outputs:
//		Number of bytes of data written to the comm port
//
//---------------------------------------------------------------------------

DWORD WriteCommBlock( PSERIAL_CONNECTION pSerialConnection, LPSTR lpByte , DWORD dwBytesToBeWritten )
{
	DWORD dwBytesWritten, i ;

	if (dwBytesToBeWritten >= pSerialConnection->dwTransmitBufferSize)
	{
		if (PRINT_DEBUG_INFO)
		{
			printf("\n\n\nINVALID PARAMETER: Cannot transmit more than %d characters in one write operation\n", 
																		pSerialConnection->dwTransmitBufferSize);
			printf("Break up writes into smaller chunks\n\n\n");
		}
		return 0;
	}
	if(!WriteFile( pSerialConnection->idComDev, lpByte, dwBytesToBeWritten, &dwBytesWritten, NULL))
	{
		if (PRINT_DEBUG_INFO)
		{
			printf("\n\n\nERROR: Writefile returned error code %d\n\n\n", GetLastError());
		}
		return 0;
	}
	if (dwBytesWritten < dwBytesToBeWritten)
	{
		if (PRINT_DEBUG_INFO)
		{
			printf("\n\n\nWRITE TIMED-OUT: Could write only %d of %d bytes\n\n\n", dwBytesWritten, dwBytesToBeWritten);
		}
	}
	if (PRINT_DEBUG_INFO)
	{
		for(i=0; i<dwBytesWritten; i++)
			printf("WROTE : 0X%x\n", *(lpByte+i));
	}
	return dwBytesWritten;
} // end of WriteCommBlock()


//---------------------------------------------------------------------------
//	Reads a character from the port.
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//	Outputs:
//		Character received from the comm port
//---------------------------------------------------------------------------
char ReadChar(PSERIAL_CONNECTION pSerialConnection)
{
	char cRecChar;
	while(!ReadCommBlock( pSerialConnection, (LPSTR) &cRecChar, 1 ));
	return cRecChar;
}

//---------------------------------------------------------------------------
//	Writes a character to the port.
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//---------------------------------------------------------------------------
void WriteChar(PSERIAL_CONNECTION pSerialConnection, char cInputChar)
{
	WriteCommBlock( pSerialConnection, (LPSTR) &cInputChar, 1 );
}

//---------------------------------------------------------------------------
//	Closes the connection to the port.
//	Inputs:
//		PSERIAL_CONNECTION
//			Pointer of the structure of comm port parameters
//---------------------------------------------------------------------------

void CloseComPort(PSERIAL_CONNECTION pSerialConnection)
{
	if( pSerialConnection->idComDev==INVALID_HANDLE_VALUE)
		return;
	// purge any outstanding reads/writes and close device handle
	PurgeComm( pSerialConnection->idComDev, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR ) ;
	CloseHandle( pSerialConnection->idComDev ) ;
} // end of CloseComPort()

